'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var mobx = require('mobx');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function deflateHelper(data) {
    function deflate(data) {
        if (!data || typeof data !== 'object')
            return data;
        if (Array.isArray(data))
            return data.map(deflate);
        // GQL object with non-root type, return object with all props deflated
        var snapshot = {};
        for (var key in data) {
            snapshot[key] = deflate(data[key]);
        }
        return snapshot;
    }
    return deflate(data);
}

function mergeHelper(store, data) {
    function merge(data) {
        if (!data || typeof data !== 'object')
            return data;
        if (Array.isArray(data))
            return data.map(merge);
        var typename = data.typename, id = data.id;
        // convert values deeply first to mobx objects as much as possible
        var snapshot = {};
        for (var key in data) {
            snapshot[key] = merge(data[key]);
        }
        // GQL object
        if (typename && store.isKnownType(typename)) {
            // GQL object with known type, instantiate or recycle MST object
            var typeDef = store.getTypeDef(typename);
            // Try to reuse instance, even if it is not a root type
            var instance = id !== undefined && store[store.getCollectionName(typename)].get(id);
            if (instance) {
                // update existing object
                instance.update(snapshot);
            }
            else {
                // create a new one
                instance = new typeDef(store.getStore, snapshot);
                if (store.isRootType(typename)) {
                    // register in the store if a root
                    store[store.getCollectionName(typename)].set(id, instance);
                }
                // instance.__setStore(store)
            }
            return instance;
        }
        else {
            // GQL object with unknown type, return verbatim
            return snapshot;
        }
    }
    return merge(data);
}

var isServer = typeof window === 'undefined';
var Query = /** @class */ (function () {
    function Query(getStore, path, method, variables, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.variables = variables;
        this.options = options;
        this.loading = false;
        this.data = undefined;
        this.error = undefined;
        this.status = undefined;
        this.clear = function () {
            _this.data = null;
        };
        this.refetch = function () {
            return Promise.resolve().then(mobx.action(function () {
                if (!_this.loading) {
                    _this.fetchResults();
                }
                _this.status = 'success';
                return _this.promise;
            }));
        };
        mobx.makeObservable(this, {
            loading: mobx.observable,
            data: mobx.observable.ref,
            error: mobx.observable,
            status: mobx.observable,
            clear: mobx.action,
            hasCache: mobx.computed
        });
        this.getStore = getStore;
        var fetchPolicy = options.fetchPolicy || 'cache-first';
        this.path = path;
        this.method = method;
        this.queryKey =
            this.path +
                JSON.stringify(__assign({ method: this.method, fetchPolicy: fetchPolicy }, variables));
        var store = this.getStore();
        if (store.ssr && !this.options.noSsr && isServer) {
            fetchPolicy = 'cache-first';
        }
        this.fetchPolicy = fetchPolicy;
        if (store.ssr && this.options.noSsr && isServer) {
            this.promise = Promise.resolve();
            return;
        }
        var inCache = store.__queryCacheData.has(this.queryKey);
        if (options.fromCache) {
            var _a = __read(options.fromCache, 2), modelName = _a[0], key = _a[1];
            var model = getCollectionName(modelName);
            if (!store[model]) {
                throw new Error('[Query] typename not found');
            }
            var fromCache = store[model].has(key);
            if (fromCache) {
                this.status = 'cache';
                this.data = store[model].get(key);
            }
        }
        if (options.initialData) {
            // cache query and response
            if (this.fetchPolicy !== 'no-cache') {
                store.__cacheResponse(this.queryKey, store.deflate(options.initialData));
            }
            var data = store.merge(options.initialData);
            this.status = 'success';
            this.loading = false;
            this.error = false;
            this.data = data;
            this.promise = Promise.resolve(data);
            return;
        }
        switch (this.fetchPolicy) {
            case 'no-cache':
            case 'network-only':
                this.fetchResults();
                break;
            case 'cache-only':
                if (!inCache) {
                    this.error = new Error("No results for query " + this.method + " found in cache, and policy is cache-only");
                    this.status = 'error';
                    this.promise = Promise.reject(this.error);
                }
                else {
                    this.useCachedResults();
                }
                break;
            case 'cache-and-network':
                if (inCache) {
                    this.useCachedResults();
                    this.refetch(); // refetch async, so that callers chaining to the initial promise should resovle immediately!
                }
                else {
                    this.fetchResults();
                }
                break;
            case 'cache-first':
                if (inCache) {
                    this.useCachedResults();
                }
                else {
                    this.fetchResults();
                }
                break;
            default:
                throw new Error('[Query] unknown fetch policy');
        }
    }
    Object.defineProperty(Query.prototype, "hasCache", {
        /**
         * this will be true and object is in the cache and loading is true TODO rename to something more clear
         */
        get: function () {
            return this.loading && this.status === 'cache';
        },
        enumerable: false,
        configurable: true
    });
    Query.prototype.fetchResults = function () {
        return __awaiter(this, void 0, void 0, function () {
            var store, promise, existingPromise;
            var _this = this;
            return __generator(this, function (_a) {
                store = this.getStore();
                this.loading = true;
                if (this.status !== 'cache')
                    this.status = 'pending';
                existingPromise = store.__promises.get(this.queryKey);
                if (existingPromise) {
                    promise = existingPromise;
                }
                else {
                    promise = store.rawRequest(this.path, this.method, this.variables);
                    store.__pushPromise(promise, this.queryKey);
                }
                promise = promise
                    .then(function (data) {
                    // cache query and response
                    if (_this.fetchPolicy !== 'no-cache') {
                        store.__cacheResponse(_this.queryKey, store.deflate(data));
                    }
                    return Promise.resolve(store.merge(data));
                })
                    .then(mobx.action(function (data) {
                    _this.status = 'success';
                    _this.loading = false;
                    _this.error = false;
                    _this.data = data;
                    return data;
                }), mobx.action(function (error) {
                    _this.status = 'error';
                    _this.loading = false;
                    _this.error = error;
                }));
                this.promise = promise;
                return [2 /*return*/];
            });
        });
    };
    Query.prototype.useCachedResults = function () {
        var store = this.getStore();
        this.status = 'success';
        this.data = store.merge(store.__queryCacheData.get(this.queryKey));
        this.promise = Promise.resolve(this.data);
    };
    Query.prototype["case"] = function (handlers) {
        return this.loading && !this.data
            ? handlers.loading()
            : this.error
                ? handlers.error(this.error)
                : handlers.data(this.data);
    };
    Query.prototype.currentPromise = function () {
        return this.promise;
    };
    Query.prototype.then = function (onfulfilled, onrejected) {
        var store = this.getStore();
        return this.promise
            .then(function (d) {
            store.__runInStoreContext(function () { return onfulfilled && onfulfilled(d); });
        })["catch"](function (e) {
            store.__runInStoreContext(function () { return onrejected && onrejected(e); });
        });
    };
    return Query;
}());

function getCollectionName(typename) {
    // Pluralize only last word (pluralize may fail with words that are
    // not valid English words as is the case with LongCamelCaseTypeNames)
    return typename.toLowerCase() + 's';
}
var MQStore = /** @class */ (function () {
    function MQStore(options, data) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.__queryCacheData = new Map();
        this.__queryCache = new Map();
        this.__promises = new Map();
        this.getStore = function () {
            return _this;
        };
        this.getCollectionName = getCollectionName;
        mobx.makeObservable(this, {
            __queryCacheData: mobx.observable,
            __queryCache: mobx.observable,
            query: mobx.action,
            deflate: mobx.action,
            rawRequest: mobx.action,
            merge: mobx.action,
            getCollectionName: mobx.action,
            __runInStoreContext: mobx.action,
            __cacheResponse: mobx.action
        });
        var request = options.request;
        if (!request) {
            throw new Error('[sact]: Sact requires either a post client or url');
        }
        if (data) {
            this.__queryCacheData = new Map(data.__queryCacheData);
        }
        this.request = request;
    }
    MQStore.prototype.merge = function (data) {
        return mergeHelper(this, data);
    };
    MQStore.prototype.deflate = function (data) {
        return deflateHelper(data);
    };
    MQStore.prototype.rawRequest = function (path, method, variables) {
        return this.request(path, method, variables);
    };
    MQStore.prototype.__cacheQuery = function (key, query) {
        this.__queryCache.set(key, query);
    };
    MQStore.prototype.query = function (path, method, variables, options) {
        if (options === void 0) { options = {}; }
        var key = path + JSON.stringify(__assign({ method: method }, variables));
        var hasQuery = this.__queryCache.get(key);
        if (hasQuery) {
            return hasQuery;
        }
        var query = new Query(this.getStore, path, method, variables, options);
        this.__cacheQuery(path + JSON.stringify(__assign({ method: method }, variables)), query);
        return query;
    };
    MQStore.prototype.__pushPromise = function (promise, queryKey) {
        var _this = this;
        this.__promises.set(queryKey, promise);
        var onSettled = function () {
            _this.__promises["delete"](queryKey);
        };
        var onError = function () {
            _this.__promises["delete"](queryKey);
        };
        promise.then(onSettled, onError);
    };
    MQStore.prototype.__runInStoreContext = function (fn) {
        return fn();
    };
    MQStore.prototype.__cacheResponse = function (key, response) {
        this.__queryCacheData.set(key, response);
    };
    return MQStore;
}());

function createStoreContext(React) {
    return React.createContext(null);
}
/**
 * note this will render your whole react tree which might not be great for performance
 */
function getDataFromTree(tree, client, renderFunction) {
    if (renderFunction === void 0) { renderFunction = require('react-dom/server').renderToStaticMarkup; }
    return __awaiter(this, void 0, void 0, function () {
        var html;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    html = renderFunction(tree);
                    if (client.__promises.size === 0) {
                        return [2 /*return*/, html];
                    }
                    return [4 /*yield*/, Promise.all(client.__promises.values())];
                case 1:
                    _a.sent();
                    return [3 /*break*/, 0];
                case 2: return [2 /*return*/];
            }
        });
    });
}
function createUseQueryHook(context, React) {
    return function (queryIn, opts) {
        if (queryIn === void 0) { queryIn = undefined; }
        if (opts === void 0) { opts = {}; }
        var store = (opts && opts.store) || React.useContext(context);
        var _a = __read(React.useState(function () {
            if (!queryIn)
                return undefined;
            return queryIn(store);
        }), 2), query = _a[0], setQuery = _a[1];
        var setQueryHelper = React.useCallback(function (newQuery) {
            setQuery(newQuery(store));
        }, [queryIn]);
        React.useEffect(function () {
            if (!queryIn || typeof queryIn === 'function')
                return; // ignore changes to initializer func
            setQueryHelper(queryIn);
        }, [queryIn]);
        if (opts.suspense) {
            if (query.status === 'pending') {
                throw query.promise;
            }
            else if (query.status === 'error') {
                throw query.error;
            }
        }
        return {
            store: store,
            loading: query ? query.loading : false,
            error: query && query.error,
            data: query && query.data,
            status: query.status,
            query: query,
            setQuery: setQueryHelper
        };
    };
}

function setTypes(self, kt, knownTypes, data) {
    if (data === void 0) { data = {}; }
    knownTypes.forEach(function (_a) {
        var _b = __read(_a, 2), key = _b[0], typeFn = _b[1];
        var type = typeFn();
        if (!type)
            throw new Error("The type provided for '" + key + "' is empty. This is probably a module loading issue");
        kt.set(key, type);
    });
    var modelKeys = Object.keys(data);
    for (var i = 0; i < modelKeys.length; i++) {
        var key = modelKeys[i];
        if (key === '__queryCacheData' || key === '__queryCache')
            continue;
        var model = self[key];
        var ids = Object.keys(data[key]);
        for (var j = 0; j < ids.length; j++) {
            var id = ids[j];
            var typeDef = kt.get(data[key][id].typename);
            if (typeDef) {
                model.set(id, new typeDef(self.getStore, data[key][id]));
            }
        }
    }
}

exports.MQStore = MQStore;
exports.Query = Query;
exports.createStoreContext = createStoreContext;
exports.createUseQueryHook = createUseQueryHook;
exports.getCollectionName = getCollectionName;
exports.getDataFromTree = getDataFromTree;
exports.setTypes = setTypes;
//# sourceMappingURL=index.js.map
